@@ -45,15 +45,14 @@ public class WordSequenceCombineAlg extends TemplateMatchingAlg
 	 */
 	private FeatureNode nextNode(FeatureNode node, int dir, String labelRegex)
 	{
-		LinkableView linkable = new LinkableView(node);
-
  		// Iterate on right links.
-		for (int i = 0; i < linkable.numLinks(dir); i++)
+ 		int nl = LinkableView.numLinks(node, dir);
+		for (int i = 0; i < nl; i++)
 		{
-			LinkView link = new LinkView(linkable.getLink(dir, i));
-			String label = link.getLabel(0);
+			FeatureNode fn = LinkableView.getLink(node, dir, i);
+			String label = LinkView.getLabel(fn, 0);
 			if (Pattern.matches(labelRegex, label))
-				return (dir == directionLeft ? link.getLeft() : link.getRight());
+				return (dir == directionLeft ? LinkView.getLeft(fn) : LinkView.getRight(fn));
 		}
 		return null;
 	}
@@ -138,6 +137,11 @@ public class WordSequenceCombineAlg extends TemplateMatchingAlg
 		// rightNode.set("start_char", leftNode.get("start_char"));
 		rightNode.get("start_char").setValue(leftNode.get("start_char").getValue());
 
+		// Create membership
+		// XXX need to set member for words in the middle too
+		// rightNode.set("memb", rightNode);
+		// leftNode.set("memb", rightNode);
+
 		// Use morphology on the right node, just in case.
 		// Err .. maybe not. What does this acheive?
 		//
