@@ -25,6 +25,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
 
+import relex.ParsedSentence;
 import relex.feature.FeatureNode;
 import relex.feature.LinkableView;
 import relex.feature.SemanticView;
@@ -42,6 +43,12 @@ public class EntityMaintainer implements Serializable
 	 */
 	public static int MAX_NUM_ENTITIES = 60;
 
+	// The list of entities
+	private EntityTagger tagger;
+
+	// Maps entity ID strings to EntityInfos
+	private Map<String,EntityInfo> iDs2Entities;
+
 	// The original sentence string
 	private String originalSentence;
 
@@ -49,12 +56,6 @@ public class EntityMaintainer implements Serializable
 	// except all entities are replaced with ID strings
 	private String convertedSentence;
 
-    // An array of EntityInfos, ordered by their order in the sentence
-    public List<EntityInfo> orderedEntityInfos;
-
-    // Maps entity ID strings to EntityInfos
-    private Map<String,EntityInfo> iDs2Entities;
-
     // Keeps track of the last entityIDIndex created
     int entityIDIndex;
 
@@ -64,101 +65,6 @@ public class EntityMaintainer implements Serializable
     // A set of integer indexes of inserted whitespace characters
     private Set<Integer> insertedWhitespaceCharIndexes;
 
-    static List<String> emolist = new ArrayList<String>();
-	
-	static
-	{
-		// Emoticons -- smiley faces, right :-)
-		// Partial list, only of the basics, taken from wikipedia
-		// This could be improved on by automatically generating
-		// these with and without noses, etc. 
-		//
-		// More generally, we should have a "bogus punctutation entity"
-		// for any sort of markup that is not recognized here.
-		//
-		// Some of these have leading or trailing whitespace ... 
-		// these are typically ones that have numbers, that's so that
-		// number expressions don't get mangled; others which might get
-		// mistaken for initials.
-		//
-		emolist.add(":-)");
-		emolist.add(":-(");
-		emolist.add(":)");
-		emolist.add(":(");
-		emolist.add(":'-)");
-		emolist.add(":')");
-		emolist.add(":D");
-		emolist.add(":-D");
-		emolist.add(":-O");
-		emolist.add(":-S");
-		emolist.add(":-$");
-		emolist.add(":-*");
-		emolist.add(":[");
-		emolist.add(":'[");
-		emolist.add(":'\\");
-		emolist.add(":-B");
-		emolist.add(":-#");
-		emolist.add(":-|");
-		emolist.add(":-&");
-		emolist.add(":-X");
-		emolist.add(":-K");
-		emolist.add(":]");
-		emolist.add(":-@");
-		emolist.add(":@");
-		emolist.add(":O]");
-		emolist.add(":d");
-		emolist.add("|-O");
-		emolist.add("%-(");
-		emolist.add("=)");
-		emolist.add("=O");
-		emolist.add(";)");
-		emolist.add(";-)");
-		emolist.add(";]");
-		emolist.add(";O]");
-		emolist.add(";O");
-		emolist.add(";D");
-		emolist.add("B-)");
-		emolist.add(" T.T "); // XXX Danger, may be initials
-		emolist.add("`:-)");
-		emolist.add(":P");
-		emolist.add("O:-)");
-		emolist.add("><");
-		emolist.add(">_<");
-		emolist.add("<_<");
-		emolist.add(">_>");
-		emolist.add(" Oo "); 
-		emolist.add(">:D");
-		emolist.add(" e.e "); // XXX Danger, may be initials
-		emolist.add("-.-*");
-		emolist.add("~.^");
-		emolist.add("(-_-)");
-		emolist.add("(-.-)");
-		emolist.add("-.-'");
-		emolist.add(" E.E "); // XXX Danger, may be initials
-		emolist.add("-.O");
-		emolist.add("*o*");
-		emolist.add("=^.^=");
-		emolist.add(" 8)");  // XXX Danger! may be legit non-smiley
-		emolist.add(" 8D "); // XXX Danger! may be legit non-smiley
-		emolist.add(">O");
-		emolist.add("(:-D");
-		emolist.add("c^:3");
-		emolist.add("~:>");
-		emolist.add("x-(");
-		emolist.add(";:^)B>");
-		emolist.add(" O.O "); // XXX Danger, may be initials
-		emolist.add(" o.o ");
-		emolist.add(" O.o ");
-		emolist.add(" o.O ");
-		emolist.add(" 8| ");   // XXX Daner, may be numerical expr.
-		emolist.add(">8V-()<");
-		emolist.add(" =3 ");  // XXX Danger, may be part of formula
-		emolist.add("-:3");
-		emolist.add(" <3 ");  // XXX Danger, may be part of formula
-		emolist.add("<><");
-		emolist.add("<@:)");
-		emolist.add(":3=");
-	}
 
 	// --------------------------------------------------------
 	/**
@@ -186,8 +92,12 @@ public class EntityMaintainer implements Serializable
 	 */
 	private void createConvertedSentence()
 	{
+		List<EntityInfo> orderedEntityInfos = tagger.tagEntities(originalSentence);
 		if ((orderedEntityInfos == null) || (orderedEntityInfos.size() == 0))
+		{
 			convertedSentence = originalSentence;
+			return;
+		}
 		convertedSentence = "";
 		int curIndex = 0;
 
@@ -284,51 +194,6 @@ public class EntityMaintainer implements Serializable
 		return id;
 	}
 
-	// --------------------------------------------------------
-	/**
-	 * Strip out emoticons, smileys :-)
-	 */
-	private void identifyEmoticons()
-	{
-		for(String emo : emolist)
-		{
-			int start = originalSentence.indexOf(emo);
-			if (start < 0) continue;
-			int end = start + emo.length();
-	
-			EntityInfo ei = new EntityInfo(originalSentence, start, end, EntityType.EMOTICON);
-			addEntity(ei);
-		}
-	}
-
-	/**
-	 * Escape parenthesis, treating them as entities.
-	 * This is needed for one reason only: the phrase markup
-	 * uses a LISP-like structure for the Penn-treebank markup,
-	 * and stray parens in the original sentence mess it up.
-	 */
-	private void escapePunct(char punct)
-	{
-		int start = 0;
-		while (true)
-		{
-			start = originalSentence.indexOf(punct, start);
-			if (start < 0) break;
-
-			EntityInfo ei = new EntityInfo(originalSentence, start, start+1, EntityType.PUNCTUATION);
-			addEntity(ei);
-			start++;
-		}
-	}
-
-	private void escapeParens()
-	{
-		escapePunct('(');
-		escapePunct(')');
-		escapePunct('[');
-		escapePunct(']');
-	}
-
 	// --------------------------------------------------------
 	
 	/**
@@ -336,24 +201,17 @@ public class EntityMaintainer implements Serializable
 	 */
 	public void addEntity(EntityInfo ei)
 	{
-		int open = 0;
-		int start = ei.getFirstCharIndex();
-		int end = ei.getLastCharIndex();
-		for (EntityInfo e: orderedEntityInfos)
-		{
-			int beg = e.getFirstCharIndex();
-			if ((open <= start) && (end <= beg))
-			{
-				int idx = orderedEntityInfos.indexOf(e);
-				orderedEntityInfos.add(idx, ei);
-				return;
-			}
-			open = e.getLastCharIndex();
+		tagger.addEntity(ei);
+	}
 
-			// If our entity overlaps with existing entities, ignore it.
-			if (start < open) return;
-		}
-		orderedEntityInfos.add(ei);
+	public void set(EntityTagger et)
+	{
+		tagger = et;
+	}
+
+	public EntityTagger getTagger()
+	{
+		return tagger;
 	}
 
 	// --------------------------------------------------------
@@ -363,33 +221,27 @@ public class EntityMaintainer implements Serializable
 	 */
 	public EntityMaintainer()
 	{
-	    
+		tagger = new EntityTaggerBasic();
 	}
 	
-	public EntityMaintainer(String _originalSentence, Collection<EntityInfo> eis)
+	public void convertSentence(String _originalSentence, Collection<EntityInfo> eis)
 	{
-		if (eis.size() > MAX_NUM_ENTITIES)
+		if (null != eis)
 		{
-			System.err.println("WARNING: Sentence had more than "
-					+ MAX_NUM_ENTITIES
-					+ ".  Ignoring extras.\nOriginal sentence:"
-					+ originalSentence);
+			if (eis.size() > MAX_NUM_ENTITIES)
+			{
+				System.err.println("WARNING: Sentence had more than "
+						+ MAX_NUM_ENTITIES
+						+ ".  Ignoring extras.\nOriginal sentence:"
+						+ originalSentence);
+			}
+			for (EntityInfo it : eis)
+			{
+				tagger.addEntity(it);
+			}
 		}
+
 		originalSentence = _originalSentence;
-		orderedEntityInfos = new ArrayList<EntityInfo>();
-
-		for (EntityInfo it : eis)
-		{
-			addEntity(it);
-		}
-
-		// Strip out emoticons, which GATE doesn't do.
-		// Emoticons confuse the parser.
-		identifyEmoticons();
-
-		// Escape parenthesis. These confuse the phrase-tree markup.
-		escapeParens();
-
 		iDs2Entities = new HashMap<String, EntityInfo>();
 		entityIDIndex = 0; // the first used index will be '1'
 
@@ -430,20 +282,19 @@ public class EntityMaintainer implements Serializable
 		int charDelta = 0;
 		try
 		{
-			for (LinkableView word = new LinkableView(leftNode);
-			     word != null;
-			     word = (word.getNext() == null ?
-			                  null : new LinkableView(word.getNext())))
+			for (FeatureNode fn = leftNode; 
+				fn != null; fn = LinkableView.getNext(fn))
 			{
+				int start = LinkableView.getStartChar(fn);
 				int previousWhiteSpace = 
-				      previouslyInsertedWhitespace(word.getStartChar());
+				      previouslyInsertedWhitespace(start);
 
-				String wordName = word.getWordString();
+				String wordName = LinkableView.getWordString(fn);
 
-				word.setStartChar(word.getStartChar() + charDelta
-						- previousWhiteSpace);
+				LinkableView.setStartChar(fn, 
+					start + charDelta - previousWhiteSpace);
 
-				// word.setExpandedStartChar(word.getExpandedStartChar()+charDelta);
+				// LinkableView.setExpandedStartChar(fn, LinkableView.getExpandedStartChar(fn)+charDelta);
 
 				if (isEntityID(wordName))
 				{
@@ -452,14 +303,14 @@ public class EntityMaintainer implements Serializable
 					charDelta += origName.length() - wordName.length();
 					try
 					{
-						SemanticView semView = new SemanticView(word.fn()
-								.get("ref"));
-						semView.setName(origName);
+						FeatureNode ref = fn.get("ref");
+						SemanticView.setName(ref, origName);
 					}
 					catch (Exception e) {}
 				}
 
-				word.setEndChar(word.getEndChar() + charDelta - previousWhiteSpace);
+				int end = LinkableView.getEndChar(fn);
+				LinkableView.setEndChar(fn, end + charDelta - previousWhiteSpace);
 				// word.setExpandedEndChar(word.getExpandedEndChar()+charDelta);
 			}
 		}
@@ -473,23 +324,22 @@ public class EntityMaintainer implements Serializable
 	}
     
 	/**
-	 * prepareSentence() -- markup parsed sentence with entity 
+	 * tagConvertedSentence() -- markup parsed sentence with entity 
 	 * information. This needs to be done before the relex algs run,
 	 * as the relex algs may use some of this information.
 	 */
-	public void prepareSentence(FeatureNode leftNode)
+	public void tagConvertedSentence(ParsedSentence parse)
 	{
-		for (LinkableView word = new LinkableView(leftNode); 
-		     word != null; 
-		     word = (word.getNext() == null ? 
-		                null : new LinkableView(word.getNext())))
+		FeatureNode fn = parse.getLeft();
+		while (fn != null)
 		{
-			String wordName = word.getWordString();
+			String wordName = LinkableView.getWordString(fn);
 			if (isEntityID(wordName))
 			{
 				EntityInfo entInfo = getEntityInfo(wordName);
-				entInfo.setProperties(word.fn());
+				entInfo.setProperties(fn);
 			}
+			fn = LinkableView.getNext(fn);
 		}
 	}
 
@@ -499,12 +349,7 @@ public class EntityMaintainer implements Serializable
      */
     public List<EntityInfo> getEntities()
     {
-        return orderedEntityInfos;
-    }
-    
-    public void setEntities(List<EntityInfo> orderedEntityInfos)
-    {
-        this.orderedEntityInfos = orderedEntityInfos;
+        return tagger.getEntities();
     }
     
     public String getConvertedSentence()
@@ -532,6 +377,8 @@ public class EntityMaintainer implements Serializable
         return iDs2Entities;
     }
 
+	/* XXX WTF? what is this for? This seems wrong to me ... */
+	@Deprecated
     public void setIDs2Entities(Map<String, EntityInfo> ds2Entities)
     {
         iDs2Entities = ds2Entities;
@@ -559,6 +406,7 @@ public class EntityMaintainer implements Serializable
 
     public String toString()
 	{
+		List<EntityInfo> orderedEntityInfos = tagger.getEntities();
 		StringBuilder sb = new StringBuilder();
 		for (EntityInfo info: orderedEntityInfos)
 		{
@@ -584,7 +432,7 @@ public class EntityMaintainer implements Serializable
             return em.originalSentence == null;
         else
             return this.originalSentence.equals(em.originalSentence) &&
-                   this.orderedEntityInfos.equals(em.orderedEntityInfos); 
+                   this.tagger.equals(em.tagger); 
     }
     
 	/**
@@ -606,7 +454,8 @@ public class EntityMaintainer implements Serializable
 			list.add(eInfo);
 			arg += 2;
 		}
-		EntityMaintainer em = new EntityMaintainer(sentence, list);
+		EntityMaintainer em = new EntityMaintainer();
+		em.convertSentence(sentence, list);
 		System.out.println(em.getConvertedSentence());
 	}
 }
